@@ -3,6 +3,8 @@ package Data::Validate::Domain;
 use strict;
 use warnings;
 
+our $VERSION = '0.13';
+
 use Net::Domain::TLD qw(tld_exists);
 
 use Exporter qw( import );
@@ -14,8 +16,6 @@ our @EXPORT = qw(
     is_domain_label
 );
 
-our $VERSION = '0.12';
-
 sub new {
     my $class = shift;
 
@@ -27,6 +27,59 @@ sub new {
 sub is_domain {
     my ( $value, $opt ) = _maybe_oo(@_);
 
+    my ( $hostname, $bits ) = _domain_labels( $value, $opt );
+
+    return unless $bits;
+
+    my $tld = $bits->[-1];
+
+    # domain_allow_single_label set to true disables this check
+    unless ( $opt->{domain_allow_single_label} ) {
+
+        # All domains have more then 1 label (neely.cx good, com not good)
+        return if @{$bits} < 2;
+    }
+
+    return $hostname if $opt->{domain_disable_tld_validation};
+
+    # If the option to enable domain_private_tld is enabled
+    # and a private domain is specified, then we return if that matches
+    if ( exists $opt->{domain_private_tld}
+        && ref( $opt->{domain_private_tld} ) ) {
+        my $lc_tld = lc($tld);
+        if ( ref( $opt->{domain_private_tld} ) eq 'HASH' ) {
+            if ( exists $opt->{domain_private_tld}->{$lc_tld} ) {
+                return $hostname;
+            }
+        }
+        else {
+            if ( $tld =~ $opt->{domain_private_tld} ) {
+                return $hostname;
+            }
+        }
+    }
+
+    # Verify domain has a valid TLD
+    return unless tld_exists($tld);
+
+    return $hostname;
+}
+
+# -------------------------------------------------------------------------------
+
+sub is_hostname {
+    my ( $value, $opt ) = _maybe_oo(@_);
+
+    my ($hostname) = _domain_labels( $value, $opt );
+
+    # We do not verify TLD for hostnames, as hostname.subhost is a valid hostname
+
+    return $hostname;
+}
+
+sub _domain_labels {
+    my ( $value, $opt ) = @_;
+
     return unless defined($value);
 
     my $length = length($value);
@@ -40,64 +93,10 @@ sub is_domain {
         return unless defined $bit;
         push( @bits, $bit );
     }
-    my $tld = $bits[-1];
 
-    #domain_allow_single_label set to true disables this check
-    unless ( defined $opt && $opt->{domain_allow_single_label} ) {
-
-        #All domains have more then 1 label (neely.cx good, com not good)
-        return if @bits < 2;
-    }
-
-    #If the option to enable domain_private_tld is enabled
-    #and a private domain is specified, then we return if that matches
-
-    if (   defined $opt
-        && exists $opt->{domain_private_tld}
-        && ref( $opt->{domain_private_tld} ) ) {
-        my $lc_tld = lc($tld);
-        if ( ref( $opt->{domain_private_tld} ) eq 'HASH' ) {
-            if ( exists $opt->{domain_private_tld}->{$lc_tld} ) {
-                return join( '.', @bits );
-            }
-        }
-        else {
-            if ( $tld =~ $opt->{domain_private_tld} ) {
-                return join( '.', @bits );
-            }
-        }
-    }
-
-    #Verify domain has a valid TLD
-    return unless tld_exists($tld);
-
-    return ( join( '.', @bits ) . $trailing_dot );
-}
-
-# -------------------------------------------------------------------------------
-
-sub is_hostname {
-    my ( $value, $opt ) = _maybe_oo(@_);
-
-    return unless defined($value);
-
-    my $length = length($value);
-    return if $length < 0 || $length > 255;
-
-    #	return is_domain_label($value) unless $value =~ /\./;  #If just a simple hostname
-
-    #Anything past here has multiple bits in it
-    my @bits;
-    foreach my $label ( split /\./, $value, -1 ) {
-        my $bit = is_domain_label( $label, $opt );
-        return unless defined $bit;
-        push( @bits, $bit );
-    }
-
-    #We do not verify TLD for hostnames, as hostname.subhost is a valid hostname
-
-    return join( '.', @bits );
+    return unless @bits;
 
+    return ( join( '.', @bits ) . $trailing_dot, \@bits );
 }
 
 sub is_domain_label {
@@ -105,7 +104,7 @@ sub is_domain_label {
 
     return unless defined($value);
 
-    #Fix Bug: 41033
+    # Fix Bug: 41033
     return if ( $value =~ /\n/ );
 
     # bail if we are dealing with more then just a hostname
@@ -113,7 +112,7 @@ sub is_domain_label {
     my $length = length($value);
     my $hostname;
     if ( $length == 1 ) {
-        if ( defined $opt && $opt->{domain_allow_underscore} ) {
+        if ( $opt->{domain_allow_underscore} ) {
             ($hostname) = $value =~ /^([0-9A-Za-z\_])$/;
         }
         else {
@@ -121,7 +120,7 @@ sub is_domain_label {
         }
     }
     elsif ( $length > 1 && $length <= 63 ) {
-        if ( defined $opt && $opt->{domain_allow_underscore} ) {
+        if ( $opt->{domain_allow_underscore} ) {
             ($hostname)
                 = $value =~ /^([0-9A-Za-z\_][0-9A-Za-z\-\_]*[0-9A-Za-z])$/;
         }
@@ -141,7 +140,7 @@ sub _maybe_oo {
         return @_[ 1, 0 ];
     }
     else {
-        return @_[ 0, 1 ];
+        return ( $_[0], ( defined $_[1] ? $_[1] : {} ) );
     }
 }
 
@@ -161,7 +160,7 @@ Data::Validate::Domain - Domain and host name validation
 
 =head1 VERSION
 
-version 0.12
+version 0.13
 
 =head1 SYNOPSIS
 
@@ -221,6 +220,12 @@ this option to a true value then C<is_domain()> will allow single label
 domains through. This is most likely to be useful in combination with
 the C<domain_private_tld> argument.
 
+=item * domain_disable_tld_validation
+
+Disables TLD validation for C<is_domain()>. This may be useful if you need to
+check domains with new gTLDs that have not yet been added to
+L<Net::Domain::TLD>.
+
 =item * domain_private_tld
 
 By default C<is_domain()> requires all domains to have a valid public TLD
@@ -362,13 +367,23 @@ Dave Rolsky <autarch@urth.org>
 
 =back
 
-=head1 CONTRIBUTOR
+=head1 CONTRIBUTORS
 
-=for stopwords David Steinbrunner
+=for stopwords David Steinbrunner Gregory Oschwald
+
+=over 4
+
+=item *
 
 David Steinbrunner <dsteinbrunner@pobox.com>
 
-=head1 COPYRIGHT AND LICENCE
+=item *
+
+Gregory Oschwald <goschwald@maxmind.com>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2016 by Neil Neely.
 
